#!/usr/bin/env python 
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/sancho/test/utest_utest.py $
$Id: utest_utest.py 25080 2004-09-13 13:41:40Z dbinger $
"""

from sancho.utest import UTest

class Test(UTest):

    def basic(self):
        print 'starting basic'
        Test._pre_count = 0
        Test._post_count = 0
        Test.a_count = 0
        Test._helper_count = 0
        class A(UTest):
            def _pre(self):
                Test._pre_count += 1
            def _post(self):
                Test._post_count += 1
            def a(self):
                Test.a_count += 1
            def _helper(self):
                Test._helper_count += 1
        A()
        assert Test._pre_count == 1
        assert Test._post_count == 1
        assert Test.a_count == 1
        assert Test._helper_count == 0


if __name__ == '__main__':
    Test()




"""sancho.code_coverage

Provides code coverage analysis.
"""

import sys, dis, types
from sets import Set

try:
    _findlinestarts = dis.findlinestarts
except AttributeError:
    # Python 2.3 needs this function
    def _findlinestarts(code):
        byte_increments = [ord(c) for c in code.co_lnotab[0::2]]
        line_increments = [ord(c) for c in code.co_lnotab[1::2]]

        lastlineno = None
        lineno = code.co_firstlineno
        addr = 0
        for byte_incr, line_incr in zip(byte_increments, line_increments):
            if byte_incr:
                if lineno != lastlineno:
                    yield (addr, lineno)
                    lastlineno = lineno
                addr += byte_incr
            lineno += line_incr
        if lineno != lastlineno:
            yield (addr, lineno)


def _find_executable_lines(code, executable_lines=None):
    if executable_lines is None:
        executable_lines = Set()
    for addr, lineno in _findlinestarts(code):
        executable_lines.add(lineno)
    for const in code.co_consts:
        if isinstance(const, types.CodeType):
            if const.co_filename == code.co_filename:
                _find_executable_lines(const, executable_lines)
    return executable_lines

class CoverageTrace:
    def __init__(self, modules):
        """modules : list of modules for which to show coverage
        """
        self.counts = {}  # {modulename: {lineno: count}}
        self.code_objects = {} # {modulename: {code_object}}
        for modulename in modules:
            self.counts[modulename] = {}
            self.code_objects[modulename] = Set()

    def _trace(self, frame, why, arg):
        if why == 'line':
            modulename = frame.f_globals.get("__name__", "<unnamed module>")
            if modulename in self.counts:
                # record line number of every trace
                lineno = frame.f_lineno
                executed_lines = self.counts[modulename]
                executed_lines[lineno] = executed_lines.get(lineno, 0) + 1
                if frame.f_code not in self.code_objects[modulename]:
                    self.code_objects[modulename].add(frame.f_code)
        return self._trace

    def runfunc(self, func, *args, **kw):
        result = None
        saved_trace = sys._getframe().f_trace
        sys.settrace(self._trace)
        try:
            result = func(*args, **kw)
        finally:
            sys.settrace(saved_trace)
        return result

    def report(self, outfile=sys.stdout, show_lines=1):
        for modulename in self.counts.keys():
            executed_lines = self.counts[modulename]
            outfile.write('%s: ' % modulename)
            if not executed_lines:
                # module as not executed as all
                outfile.write("not executed\n")
                continue
            outfile.write('\n')

            filename = sys.modules[modulename].__file__
            if filename[-4:] == ".pyc" or filename[-4:] == ".pyo":
                filename = filename[:-4] + '.py'
            elif filename[-5:] == ".ptlc":
                filename = filename[:-1]

            try:
                lines = open(filename).readlines()
            except IOError, exc:
                sys.stderr.write("%s: Could not open %r: %s - skipping\n" %
                                 ("trace", filename, exc.strerror))
                continue

            executable_lines = Set()
            for code_object in self.code_objects[modulename]:
                _find_executable_lines(code_object, executable_lines)

            for lineno, line in enumerate(lines):
                lineno += 1
                if lineno in executed_lines:
                    outfile.write('%5d: ' % executed_lines[lineno])
                else:
                    if lineno in executable_lines:
                        outfile.write('@%-5d>' % lineno)
                    else:
                        outfile.write(' '*7)
                outfile.write(line)


def report_coverage(modules, func, *args, **kwargs):
    ct = CoverageTrace(modules)
    rv = ct.runfunc(func, *args, **kwargs)
    ct.report()
    return rv



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/sancho/utest.py $
$Id: utest.py 26714 2005-04-28 19:59:19Z dbinger $

a lean unit test framework
"""

import sys, cStringIO, traceback, inspect

class UTest:

    def __init__(self):
        """
        Run each method of self whose name does not start with an underscore
        as a test.
        Run self._pre() before each test method.
        Run self._post() after each test method.
        Trap all output to stdout, except when an exception is raised
        in the method being called.
        If an exception is raised in a method call, print any generated
        output and a traceback, and proceed to the next method call.
        """

        def run(method, name):
            """(method:MethodType, name:str)
            Run method, label output using name.
            The method runs with stdout redirected to a cStringIO.
            If it runs without raising an exception, no output is generated.
            If an exception is generated, it prints the accumulated output
            and a traceback.
            """
            old_stdout = sys.stdout
            sys.stdout = cStringIO.StringIO()
            try:
                print '.-- %s' % name
                try:
                    method(self)
                except KeyboardInterrupt:
                    raise
                except:
                    # print traceback, restore stdout
                    data = ''.join(traceback.format_exception(*sys.exc_info()))
                    for line in data.rstrip().split('\n'):
                        sys.stdout.write('|' + line + '\n')
                    sys.__stdout__.write(sys.stdout.getvalue())
            finally:
                sys.stdout = old_stdout
        names = [name for name in dir(self.__class__)
                 if not name.startswith('_')]
        names.sort()
        printed = False
        for name in names:
            check = getattr(self.__class__, name)
            if inspect.ismethod(check):
                if not printed:
                    print ("%s: %s:" % (inspect.getsourcefile(check),
                                        self.__class__.__name__))
                    printed = True
                run(self.__class__._pre, '%s: _pre' % name)
                run(check, name)
                run(self.__class__._post, '%s: _post' % name)

    def _pre(self):
        """
        Executed before each test method.
        Override as needed.
        """

    def _post(self):
        """
        Executed after each test method.  Override as needed.
        This default removes any attributes whose names do not start with '_'
        """
        for name in self.__dict__.keys():
            if not name.startswith('_'):
                delattr(self, name)


def raises(exc, func, *args, **kwargs):
    """(exception, func, *args, **kwargs) -> bool

    Return True if func(*args, **kwargs) raises exc.
    """
    try:
       func(*args, **kwargs)
    except exc:
        return True
    else:
        return False




#!/usr/bin/env python 
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/sancho/test/utest_template.py $
$Id: utest_template.py 25080 2004-09-13 13:41:40Z dbinger $
"""

from sancho.utest import UTest

class Test(UTest):

    def _pre(self):
        """Setup code for each test method."""

    def _post(self):
        """Shutdown code for each test method."""

    def basic(self):
        """A test method
        Output to stdout is trapped, unless an exception is raised here.
        """

if __name__ == '__main__':
    Test()



#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/sancho/convert.py $
$Id: convert.py 25081 2004-09-13 13:48:12Z dbinger $

This script is designed to assist in the conversion of
the pre-2.0 Sancho unittest scripts into the new
Sancho utest format.  Some additional hand editing of
the output may be required, but this takes care of the
simple patterns.
"""

import re
import sys

def test_val(line, write,
             pattern=re.compile("(?P<indentation>\s*)"
                                "(?P<function_call>self.test_val)"
                                "(\()"
                                "(?P<quote>['\"])"
                                "(?P<arg1>.*)"
                                "(?P=quote),"
                                "\s*(?P<arg2>.*)"
                                "(\))"
                                )):
    match = pattern.match(line)
    if not match:
        return False
    write("%sassert %s == %s\n" % (match.group('indentation'),
                                   match.group('arg1'),
                                   match.group('arg2')))
    return True

def test_seq(line, write,
             pattern=re.compile("(?P<indentation>\s*)"
                                "(?P<function_call>self.test_seq)"
                                "(\()"
                                "(?P<quote>['\"])"
                                "(?P<arg1>.*)"
                                "(?P=quote),"
                                "\s*(?P<arg2>.*)"
                                "(\))"
                                )):
    match = pattern.match(line)
    if not match:
        return False
    write("%sassert %s == %s\n" % (match.group('indentation'),
                                               match.group('arg1'),
                                               match.group('arg2')))
    return True

def test_set(line, write,
             pattern=re.compile("(?P<indentation>\s*)"
                                "(?P<function_call>self.test_set)"
                                "(\()"
                                "(?P<quote>['\"])"
                                "(?P<arg1>.*)"
                                "(?P=quote),"
                                "\s*(?P<arg2>.*)"
                                "(\))"
                                )):
    match = pattern.match(line)
    if not match:
        return False
    write("%sassert Set(%s) == Set(%s)\n" % (match.group('indentation'),
                                             match.group('arg1'),
                                             match.group('arg2')))
    return True


def test_stmt(line, write,
              pattern=re.compile("(?P<indentation>\s*)"
                                 "(?P<function_call>self.test_stmt)"
                                 "(\()"
                                 "(?P<quote>['\"])"
                                 "(?P<arg1>.*)"
                                 "(?P=quote)"
                                 "(\))"
                                 )):
    match = pattern.match(line)
    if not match:
        return False
    write("%s%s\n" % (match.group('indentation'), match.group('arg1')))
    return True

def test_true(line, write,
              pattern=re.compile("(?P<indentation>\s*)"
                                 "(?P<function_call>self.test_true)"
                                 "(\()"
                                 "(?P<quote>['\"])"
                                 "(?P<arg1>.*)"
                                 "(?P=quote)"
                                 "(\))"
                                 )):
    match = pattern.match(line)
    if not match:
        return False
    write("%sassert %s\n" % (match.group('indentation'),
                             match.group('arg1')))
    return True

def test_bool_true(line, write,
                   pattern=re.compile("(?P<indentation>\s*)"
                                      "(?P<function_call>self.test_bool)"
                                      "(\()"
                                      "(?P<quote>['\"])"
                                      "(?P<arg1>.*)"
                                      "(?P=quote)"
                                      "(\))"
                                      )):
    match = pattern.match(line)
    if not match:
        return False
    write("%sassert %s\n" % (match.group('indentation'),
                             match.group('arg1')))
    return True


def test_false(line, write,
               pattern=re.compile("(?P<indentation>\s*)"
                                  "(?P<function_call>self.test_false)"
                                  "(\()"
                                  "(?P<quote>['\"])"
                                  "(?P<arg1>.*)"
                                  "(?P=quote)"
                                  "(\))"
                                  )):
    match = pattern.match(line)
    if not match:
        return False
    write("%sassert not %s\n" % (match.group('indentation'),
                                 match.group('arg1')))
    return True

def test_exc(line, write,
             pattern=re.compile("(?P<indentation>\s*)"
                                "(?P<function_call>self.test_exc)"
                                "(\()"
                                "(?P<quote>['\"])"
                                "(?P<arg1>.*)"
                                "(?P=quote),"
                                "\s*(?P<arg2>.*)"
                                "(\))"
                                )):
    match = pattern.match(line)
    if not match:
        return False
    write(("%stry:\n"
           "%s    %s\n"
           "%s    assert 0\n"
           "%sexcept %s: pass\n") % (
        match.group('indentation'),
        match.group('indentation'),
        match.group('arg1'),
        match.group('indentation'),
        match.group('indentation'),
        match.group('arg2')))
    return True

def run_scenarios(line, write,
                  pattern=re.compile("(?P<indentation>\s*)run_scenarios")):

    match = pattern.match(line)
    if not match:
        return False
    for test in TESTS:
        write("%s%s()\n" % (match.group('indentation'),
                            test))
    return True

TESTS = []

def class_testscenario(line, write,
                       pattern=re.compile("class "
                                          "(?P<class_name>\S+)"
                                          "\s?\("
                                          "(TestScenario)"
                                          "\):"
                                          )):
    match = pattern.match(line)
    if not match:
        return False
    write("class %s (UTest):\n" % match.group('class_name'))
    global TESTS
    TESTS.append(match.group('class_name'))
    return True

def parse_args(line, write,
               pattern=re.compile("\s*\(scenarios, options\)")):
    if pattern.match(line):
        return True
    else:
        return False

def coverage_line(line, write,
                  pattern=re.compile("\s*\options.show_coverage")):
    if pattern.match(line):
        return True
    else:
        return False

def from_sancho(line, write, pattern=re.compile("from sancho")):
    if pattern.match(line):
        write("from sancho.utest import UTest\n")
        return True
    else:
        return False

def tested_modules(line, write, pattern=re.compile("tested_modules")):
    if pattern.match(line):
        return True
    else:
        return False

def test_cases(line, write, pattern=re.compile("test_cases")):
    if pattern.match(line):
        return True
    else:
        return False

def def_setup(line, write,
              pattern=re.compile("(?P<indentation>\s*)"
                                 "def (?P<function_name>setup)"
                                 "(?P<rest>.*)"
                                 )):
    match = pattern.match(line)
    if not match:
        return False
    for test in TESTS:
        write("%sdef _pre%s\n" % (match.group('indentation'),
                                  match.group('rest')))
    return True

def def_shutdown(line, write,
                 pattern=re.compile("(?P<indentation>\s*)"
                                    "def (?P<function_name>shutdown)"
                                    "(?P<rest>.*)"
                                 )):
    match = pattern.match(line)
    if not match:
        return False
    for test in TESTS:
        write("%sdef _post%s\n" % (match.group('indentation'),
                                  match.group('rest')))
    return True

def convert(in_file=sys.stdin, out_file=sys.stdout):
    for line in in_file.readlines():
        for function in (from_sancho,
                         tested_modules,
                         test_val,
                         test_stmt,
                         test_true,
                         test_bool_true,
                         test_false,
                         test_exc,
                         test_cases,
                         test_seq,
                         test_set,
                         def_setup,
                         def_shutdown,
                         class_testscenario,
                         parse_args,
                         coverage_line,
                         run_scenarios):
            if function(line, out_file.write):
                break
        else:
            out_file.write(line)


def convert_main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.set_description("Convert sancho TestScenario file to UTest format.")
    parser.add_option(
        '-i', '--in', dest="in_file", default=None,
        help="TestScenario file to be converted. (default stdin)")
    parser.add_option(
        '-o', '--out', dest="out_file", default=None,
        help="Output file (default stdout)")
    (options, args) = parser.parse_args()
    if options.in_file:
        infile = open(options.in_file, 'r')
    else:
        infile = sys.stdin
    if options.out_file:
        outfile = open(options.out_file, 'w')
    else:
        outfile = sys.stdout
    convert(infile, outfile)

if __name__ == '__main__':
    convert_main()





#!/usr/bin/env python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/sancho/setup.py $
$Id: setup.py 27128 2005-08-01 21:00:30Z dbinger $
"""
from distutils.core import setup

setup(name = "Sancho",
      version = "2.1",
      description = "A simple unit testing framework",
      author = "CNRI",
      author_email = "webmaster@mems-exchange.org",
      url = "http://www.mems-exchange.org/software/sancho/",
      license = "see LICENSE.txt",
      package_dir = {"sancho": "."},
      packages = ["sancho"],
      scripts = ["urun.py"],
      platforms = ['Python 2.3'],
     )



#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/sancho/urun.py $
$Id: urun.py 26920 2005-06-16 17:26:14Z rmasse $

Command line tool for running utest_*py scripts.
"""

import os
from optparse import OptionParser
from sancho import code_coverage

def run_utests(filenames):
    for filename in filenames:
        execfile(filename, {'__name__': '__main__', '__file__': filename})

def main():
    usage = 'usage: %prog [-c <modulename>] [filename | directory]'
    parser = OptionParser(usage=usage)
    parser.set_description('Run a set of utest modules.  Each file named '
                           'will be executed.  If a directory name is '
                           'provided then all files under that directory '
                           'that match the pattern utest_*.py will executed.')
    parser.add_option('-c', '--show-coverage',
                      dest='module', default=[], action='append',
                      help='After running tests, show code coverage of the '
                           'named module (provide the name, not the '
                           'filename).')
    (options, args) = parser.parse_args()
    matches = []
    args = args or ['.']
    for pat in args:
        if os.path.isdir(pat):
            for dirpath, dirnames, filenames in os.walk(pat):
                for name in filenames:
                    if name.startswith('utest_') and name.endswith('.py'):
                        matches.append(os.path.join(dirpath, name))
        else:
            matches.append(pat)
    matches.sort()
    if options.module:
        code_coverage.report_coverage(options.module, run_utests, matches)
    else:
        run_utests(matches)


if __name__ == '__main__':
    main()
